﻿--[[
	Project: HideParty
	Author: Erfolg

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>. 
]]
HideParty = LibStub("AceAddon-3.0"):NewAddon("HideParty", "AceConsole-3.0")

local HIDEPARTY_VERSION = GetAddOnMetadata("HideParty", "X-Curse-Packaged-Version")

local L = LibStub("AceLocale-3.0"):GetLocale("HideParty")

local options = {
	name = "HideParty",
	handler = HideParty,
	type = 'group',
	args = {
		descrp = {
			type = "description",
			name = L["Description"],
		},
		hide = {
			name = L["General"],
			handler = HideParty,
			type = 'group',
			args = {
				player = {
					type = 'toggle',
					name = L["Player"],
					desc = L["PlayerDesc"],
					get = function(info) return HideParty.db.profile.player end,
					set = function(info, value)
						HideParty.db.profile.player = value
						HideParty:UpdateFrames()
						end,
				},
				party = {
					type = 'toggle',
					name = L["Party"],
					desc = L["PartyDesc"],
					get = function(info) return HideParty.db.profile.party end,
					set = function(info, value)
						HideParty.db.profile.party = value
						HideParty:UpdateFrames()
						end,
				},
				raid = {
					type = 'toggle',
					name = L["Raid"],
					desc = L["RaidDesc"],
					get = function(info) return HideParty.db.profile.raid end,
					set = function(info, value)
						HideParty.db.profile.raid = value
						HideParty:UpdateFrames()
						end,
				},
				target = {
					type = 'toggle',
					name = L["Target"],
					desc = L["TargetDesc"],
					get = function(info) return HideParty.db.profile.target end,
					set = function(info, value)
						HideParty.db.profile.target = value
						HideParty:UpdateFrames()
						end,
				},
				focus = {
					type = 'toggle',
					name = L["Focus"],
					desc = L["FocusDesc"],
					get = function(info) return HideParty.db.profile.focus end,
					set = function(info, value)
						HideParty.db.profile.focus = value
						local frame = "focus"
						HideParty:UpdateFrames()
						end,
				},
				castbar = {
					type = 'toggle',
					name = L["CastBar"],
					desc = L["CastBarDesc"],
					get = function(info) return HideParty.db.profile.castbar end,
					set = function(info, value)
						HideParty.db.profile.castbar = value
						HideParty:UpdateFrames()
						end,
				},
				aura = {
					type = 'toggle',
					name = L["Buffs"],
					desc = L["BuffsDesc"],
					get = function(info) return HideParty.db.profile.aura end,
					set = function(info, value)
						HideParty.db.profile.aura = value
						HideParty:UpdateFrames()
						end,
				},
				runebar = {
					type = 'toggle',
					name = L["RuneBar"],
					desc = L["RuneBarDesc"],
					get = function(info) return HideParty.db.profile.runebar end,
					set = function(info, value)
						HideParty.db.profile.runebar = value
						HideParty:UpdateFrames()
						end,
				},
				altpower = {
					type = 'toggle',
					name = L["AltPower"],
					desc = L["AltPowerDesc"],
					get = function(info) return HideParty.db.profile.altpower end,
					set = function(info, value)
						HideParty.db.profile.altpower = value
						HideParty:UpdateFrames()
						end,
				},
			},
		},
	},
}

local defaults = {
	profile = {
		player = true,
		party = true,
		raid = true,
		target = true, 
		focus = true,
		castbar = false,
		aura = false,
		runebar = true,
		altpower = false,
	},
}

local show = {}
local hide = {}
local hidden = {}

function HideParty:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("HidePartyDB", defaults, "Default")
	self.db.RegisterCallback(self, "OnProfileChanged", "UpdateFrames")
	self.db.RegisterCallback(self, "OnProfileCopied", "UpdateFrames")
	self.db.RegisterCallback(self, "OnProfileReset", "UpdateFrames")
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("HideParty", options)
	self.Options = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("HideParty")
end

function HideParty:OnEnable()
	self:RegisterChatCommand("HP", "SlashCommand")
	self:RegisterChatCommand("HIDEPARTY", "SlashCommand")
	self:Print(HIDEPARTY_VERSION)
	self:UpdateFrames()
end

function HideParty:OnDisable()
	
end

function HideParty:UpdateFrames()
	for frame in pairs(show) do
		if self:IsEnabled() and self.db.profile[frame] then
			if not hidden[frame] then
				hidden[frame] = true
				hide[frame](self)
			end
		else
			if hidden[frame] then
				hidden[frame] = nil
				show[frame](self)
			end
		end
	end
end

local function hook_playerframe()
	hooksecurefunc("PlayerFrame_HideVehicleTexture",function()
		if hidden["runebar"] then
			hide["runebar"]()
		end
	end)
	hook_playerframe = nil
end

local function hook_raidmanager()
	hooksecurefunc("CompactRaidFrameManager_UpdateShown",function()
		if hidden["raid"] then
			hide["raid"]()
		end
	end)
	hook_raidmanager = nil
end

local function hook_compactparty()
	hooksecurefunc("CompactPartyFrame_UpdateShown",function()
		if hidden["party"] then
			hide["party"]()
		end
	end)
	hook_compactparty = nil
end

function hide:player()
	PlayerFrame:Hide()
end

function show:player()
	PlayerFrame:Show()
end

function hide:party()
	for i = 1, 4 do
		local frame = _G["PartyMemberFrame"..i]
		frame:UnregisterAllEvents()
		frame:Hide()
		frame.Show = function() end
	end
	
	UIParent:UnregisterEvent("GROUP_ROSTER_UPDATE")
end

function show:party()
	for i = 1, 4 do
		local frame = _G["PartyMemberFrame"..i]
		frame.Show = nil
		frame:GetScript("OnLoad")(frame)
		frame:GetScript("OnEvent")(frame, "GROUP_ROSTER_UPDATE")
		
		PartyMemberFrame_UpdateMember(frame)
	end

	UIParent:RegisterEvent("GROUP_ROSTER_UPDATE")
end

local compact_raid
function hide:raid()
	CompactRaidFrameManager:UnregisterEvent("GROUP_ROSTER_UPDATE")
	CompactRaidFrameManager:UnregisterEvent("PLAYER_ENTERING_WORLD")
	CompactRaidFrameManager:Hide()
	compact_raid = CompactRaidFrameManager_GetSetting("IsShown")
	if compact_raid and compact_raid ~= "0" then 
		CompactRaidFrameManager_SetSetting("IsShown", "0")
	end
	if hook_raidmanager then
		hook_raidmanager()
	end
end

function show:raid()
	CompactRaidFrameManager:RegisterEvent("GROUP_ROSTER_UPDATE")	
	CompactRaidFrameManager:RegisterEvent("PLAYER_ENTERING_WORLD")
	if GetDisplayedAllyFrames then
		if GetDisplayedAllyFrames() == "raid" then
			CompactRaidFrameManager:Show()
		end
	elseif GetNumRaidMembers() > 0 then
		CompactRaidFrameManager:Show()
	end
	if compact_raid and compact_raid ~= "0" then
		CompactRaidFrameManager_SetSetting("IsShown", "1")
	end
end

function hide:target()
	TargetFrame:UnregisterAllEvents()
	TargetFrame:Hide()

	ComboFrame:UnregisterAllEvents()
end

function show:target()
	TargetFrame:GetScript("OnLoad")(TargetFrame)

	ComboFrame:GetScript("OnLoad")(ComboFrame)
end

function hide:focus()
	FocusFrame:UnregisterAllEvents()
	FocusFrame:Hide()
end

function show:focus()
	FocusFrame:GetScript("OnLoad")(FocusFrame)
end

function hide:castbar()
	CastingBarFrame:UnregisterAllEvents()
	PetCastingBarFrame:UnregisterAllEvents()
end

function show:castbar()
	CastingBarFrame:GetScript("OnLoad")(CastingBarFrame)
	PetCastingBarFrame:GetScript("OnLoad")(PetCastingBarFrame)
end

function hide:runebar()
	if hook_playerframe then
		hook_playerframe()
	end
	RuneFrame:UnregisterAllEvents()
	RuneFrame:Hide()
end

function show:runebar()
	local _,class = UnitClass("player")
	if class == "DEATHKNIGHT" then
		RuneFrame:Show()
	end
	RuneFrame:GetScript("OnLoad")(RuneFrame)
	RuneFrame:GetScript("OnEvent")(RuneFrame, "PLAYER_ENTERING_WORLD")
end

function hide:aura()
	BuffFrame:Hide()
	TemporaryEnchantFrame:Hide()
	ConsolidatedBuffs:Hide()
	BuffFrame:UnregisterAllEvents()
end

function show:aura()
	BuffFrame:Show()
	if GetCVarBool("consolidateBuffs") then
		ConsolidatedBuffs:Show()
	end
	TemporaryEnchantFrame:Show()
	BuffFrame:RegisterEvent("UNIT_AURA")
end

function hide:altpower()
	PlayerPowerBarAlt:UnregisterAllEvents()
	PlayerPowerBarAlt:Hide()
end

function show:altpower()
	PlayerPowerBarAlt:GetScript("OnLoad")(PlayerPowerBarAlt)
	UnitPowerBarAlt_UpdateAll(PlayerPowerBarAlt)
end

function HideParty:SlashCommand(input)
	if input == "version" then
		self:Print(HIDEPARTY_VERSION)
	elseif input == "options" then
		InterfaceOptionsFrame_OpenToCategory(self.Options)
	else
		self:Print("/hp options")
		self:Print("/hp version")
	end
end